@@ -31,13 +31,13 @@ RPB_FUNCTION_1(Rcpp::CharacterVector, METHOD(getMemberNames), Rcpp::XPtr<GPB::De
     Rcpp::CharacterVector res(nfields + ntypes + nenums);
     int j = 0;
     for (int i = 0; i < nfields; i++, j++) {
-        res[j] = desc->field(i)->name();
+        res[j] = std::string(desc->field(i)->name());
     }
     for (int i = 0; i < ntypes; i++, j++) {
-        res[j] = desc->nested_type(i)->name();
+        res[j] = std::string(desc->nested_type(i)->name());
     }
     for (int i = 0; i < nenums; i++, j++) {
-        res[j] = desc->enum_type(i)->name();
+        res[j] = std::string(desc->enum_type(i)->name());
     }
     return (res);
 }
@@ -59,17 +59,17 @@ RPB_FUNCTION_1(Rcpp::List, METHOD(as_list), Rcpp::XPtr<GPB::Descriptor> desc) {
     for (int i = 0; i < nfields; cnt++, i++) {
         const GPB::FieldDescriptor* fd = desc->field(i);
         res[cnt] = S4_FieldDescriptor(fd);
-        names[cnt] = fd->name();
+        names[cnt] = std::string(fd->name());
     }
     for (int i = 0; i < ntypes; cnt++, i++) {
         const GPB::Descriptor* d = desc->nested_type(i);
         res[cnt] = S4_Descriptor(d);
-        names[cnt] = d->name();
+        names[cnt] = std::string(d->name());
     }
     for (int i = 0; i < nenums; cnt++, i++) {
         const GPB::EnumDescriptor* ed = desc->enum_type(i);
         res[cnt] = S4_EnumDescriptor(ed);
-        names[cnt] = ed->name();
+        names[cnt] = std::string(ed->name());
     }
     res.names() = names;
 
@@ -120,7 +120,7 @@ RPB_FUNCTION_1(S4_FileDescriptor, METHOD(fileDescriptor), Rcpp::XPtr<GPB::Descri
 }
 
 RPB_FUNCTION_2(std::string, METHOD(name), Rcpp::XPtr<GPB::Descriptor> d, bool full) {
-    return full ? d->full_name() : d->name();
+    return std::string(full ? d->full_name() : d->name());
 }
 
 RPB_FUNCTION_2(S4_Message, METHOD(readMessageFromFile), Rcpp::XPtr<GPB::Descriptor> desc,
@@ -225,6 +225,65 @@ RPB_FUNCTION_3(S4_Message, METHOD(readASCIIFromConnection), Rcpp::XPtr<GPB::Desc
     return (S4_Message(message));
 }
 
+RPB_FUNCTION_2(S4_Message, METHOD(readJSONFromString), Rcpp::XPtr<GPB::Descriptor> desc,
+               std::string input) {
+#ifdef PROTOBUF_JSON_UTIL
+    GPB::Message* message = PROTOTYPE(desc);
+    if (!message) {
+        Rcpp::stop("could not call factory->GetPrototype(desc)->New()");
+    }
+    #if GOOGLE_PROTOBUF_VERSION < 4022000
+        GPB::util::Status status = GPB::util::JsonStringToMessage(input, message);
+    #else
+        absl::Status status = GPB::util::JsonStringToMessage(input, message);
+    #endif
+    if (!status.ok()) {
+        Rcpp::stop(status.ToString().c_str());
+    }
+    return (S4_Message(message));
+#else
+    Rcpp::stop(
+        "The protobuf library you are using is too old for using JSON utility functions, "
+        "please upgrade to version 3 or above.");
+#endif
+}
+
+RPB_FUNCTION_2(S4_Message, METHOD(readJSONFromConnection), Rcpp::XPtr<GPB::Descriptor> desc,
+               int conn_id) {
+#ifdef PROTOBUF_JSON_UTIL
+    std::string json_string;
+    RconnectionCopyingInputStream wrapper(conn_id);
+    GPB::io::CopyingInputStreamAdaptor stream(&wrapper);
+    const void* buffer;
+    int size;
+    while (stream.Next(&buffer, &size)) {
+        json_string.append((char *) buffer, size);
+    }
+    if (wrapper.Failure()) {
+        Rcpp::stop("Could not read JSON protocol buffer.");
+    }
+
+    /* create a prototype of the message we are going to read */
+    GPB::Message* message = PROTOTYPE(desc);
+    if (!message) {
+        Rcpp::stop("could not call factory->GetPrototype(desc)->New()");
+    }
+    #if GOOGLE_PROTOBUF_VERSION < 4022000
+        GPB::util::Status status = GPB::util::JsonStringToMessage(json_string, message);
+    #else
+        absl::Status status = GPB::util::JsonStringToMessage(json_string, message);
+    #endif
+    if (!status.ok()) {
+        Rcpp::stop(status.ToString().c_str());
+    }
+    return (S4_Message(message));
+#else
+    Rcpp::stop(
+        "The protobuf library you are using is too old for using JSON utility functions, "
+        "please upgrade to version 3 or above.");
+#endif
+}
+
 RcppExport SEXP Descriptor_getField(SEXP pointer, SEXP name) {
     GPB::FieldDescriptor* field_desc = (GPB::FieldDescriptor*)0;
     BEGIN_RCPP
